@@ -1,23 +1,24 @@
 ;;; deb-view.el: View Debian package archive files with tar-mode.
 
 ;; Author:  Rick Macdonald (rickm@vsl.com)
-;; Version: 1.4
+;; Version: 1.7
 ;; Latest version: http://www.cuug.ab.ca/~macdonal/deb-view.el
+;; Latest version: http://www.cuug.ab.ca/~macdonal/deb-view.el.gz
 
 ;; This file is not part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; deb-view is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; GNU Emacs is distributed in the hope that it will be useful,
+;; deb-view is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; along with deb-view; see the file COPYING.  If not, write to the
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
@@ -26,9 +27,18 @@
 ;; deb-view presents the contents of debian package archive files for
 ;; viewing. The viewing is done with emacs tar-mode, with a few
 ;; enhancements for viewing compressed files and HTML files, and
-;; formatting man pages.
+;; formatted man pages.
 ;; The normal editing and saving features of tar-mode are not supported
-;; by deb-view. Don't even try (I didn't).
+;; by deb-view.
+
+;; deb-view includes a command called deb-find which requires that you
+;; have the debian distribution directories on a local or mounted
+;; filesystem. Give it a string or regular expression and it presents a
+;; buffer of matching deb file names.  Click with the middle mouse button
+;; or press RETURN (or ^C^C) and it launches deb-view on the selected
+;; file. deb-find can be configured to use locate or find, or any other
+;; external command. The find method passes your search specification to
+;; egrep, whereas the locate method uses your string directly.
 
 ;; deb-view extracts the control.tar.gz and data.tar.gz files from debian
 ;; package and presents two buffers in tar-mode. See tar-mode for info.
@@ -41,7 +51,9 @@
 ;; For new-style .deb files (2.0), dpkg-deb isn't used. Therefore
 ;; deb-view should work on any platform with the ar command, although
 ;; "ar -p" doesn't seem to work for .deb files on Solaris 2.4 and 2.5.
-;; It works on SGI's IRIX 6.1 and 6.2, and Linux, of course.
+;; It works on Solaris 2.6, SGI's IRIX 6.1 and 6.2, and Linux, of course.
+
+;; deb-view apparently doesn't work with emacs 20.x.
 
 ;; Old-style .deb files require the dpkg-deb program. I don't know how to
 ;; extract control.tar.gz from these deb files, so you only get to see
@@ -52,34 +64,86 @@
 
 ;;; Installation:
 
-;; Put this file somewhere where Emacs can find it (i.e., in one of the paths
-;; in your `load-path'), `byte-compile-file' it, and put in your ~/.emacs:
-;; (load "deb-view")
-;;
-;; If you're not very familiar with emacs customization, try this simpler
-;; approach:
+;; 1) Quick test to see if you like deb-view.
+
 ;; Put this file in your home directory, and call it deb-view.el.
 ;; Start up emacs and do the following:
 ;;      ESCAPE x load-file RETURN ~/deb-view.el RETURN
-;; If you like it, and want it available always, then add this line to your
-;; ~/.emacs file (or create ~/.emacs if you don't have one):
-;; (load "~/deb-view.el")
+;; Then, view a deb file with CTRL-d in Dired mode,
+;; or execute:
+;;      ESCAPE x deb-view RETURN {/full/path/of/file.deb} RETURN
+;; or execute:
+;;      ESCAPE : (setq deb-find-directory "/your/debian/directory") RETURN
+;;      ESCAPE x deb-find RETURN {deb-file-search-string} RETURN
+;;      and select a deb file to view with RETURN or middle mouse button in
+;;      the search results buffer that is created. Exit this buffer with "q".
+
+;; 2) Permanent installation.
+
+;; When installed this way, all find-file operations (such as "f" or "v" in
+;; dired-mode) will automatically recognize debian files and load deb-view
+;; when required.
+
+;; Put this file somewhere where Emacs can find it (i.e., in one of the paths
+;; in your `load-path'), `byte-compile-file' it, and put the following six
+;; lines (with semi-colons removed) in your ~/.emacs file (or create ~/.emacs
+;; if you don't have one):
+;; (autoload 'deb-find            "deb-view" "Dired Debian Archive File Finder" t)
+;; (autoload 'deb-view-mode       "deb-view" "Debian Archive File Mode" t)
+;; (autoload 'deb-view            "deb-view" "Dired Debian Archive File Viewer" t)
+;; (autoload 'deb-view-dired-view "deb-view" "Dired Debian Archive File Viewer" t)
+;; (setq auto-mode-alist (append '(("\\.deb$" . deb-view-mode)) auto-mode-alist))
+;; (define-key dired-mode-map     "\C-d"     'deb-view-dired-view) 
+
+;; If you're not very familiar with emacs customization, here is a simpler
+;; approach. Add this line to your ~/.emacs file (or create ~/.emacs if you
+;; don't have one):
+;;     (load "~/deb-view.el")
+;; Or, if you can put deb-view into your load-path (execute
+;; "^h v load-path RETURN" to see your load-path setting)
+;; then just add the following to your ~./emacs file:
+;;     (require 'deb-view)
 
 ;; deb-view tries to be unobtrusive, but does the following:
-;; binds ^d in dired to deb-view-dired-view.
+;; binds ^d in dired to deb-view-dired-view. (This is somewhat obsolete).
 ;; binds q, N, W, and re-binds v in tar-mode. These functions are then
-;;       available in tar-mode on real tar files.
+;;       available in standard tar-mode on real tar files.
+
+;; 3) Configuration
+
+;; deb-find has two variables to set. deb-find-method can be "locate" or
+;; "find". Any other value will be assumed to be an external script or
+;; program that you supply. If you set deb-find-method to "find" then you
+;; must also set deb-find-directory to the directory containing the
+;; debian distribution. The find command starts at this point. I originally
+;; used the locate option, but contrary to the man page it doesn't seem to
+;; understand even simple regular expressions. I prefer the find option. It
+;; uses egrep and therefore understands complex regular expressions.
+;; You might want to bind deb-find to a special key. I use ^C^D like this:
+;; (define-key ctl-x-map "\C-d" 'deb-find)
+;; Note that this key is normally the brief list-directory command, but I
+;; never used it anyway.
 
 
 ;;; Usage:
 
-;; In dired, press ^d on the dired line of the .deb file to view.
-;; Or, execute: ESC x deb-view RET, and enter the .deb file name
+;; In dired, press f or e on the dired line of the .deb file to view.
+;; You can also use ^d, but this is somewhat obsolete now and might
+;; be removed in the future.
+
+;; Or, execute: ESCAPE x deb-view RETURN, and enter the .deb file name
 ;; at the prompt.
 
+;; Or, execute: ESCAPE x deb-find RETURN, and enter any substring of a
+;; deb file name to search for. A buffer of matches is created.
+;; Launch deb-view by selecting a deb file with the middle mouse button,
+;; or RETURN or ^c^c. Exit this buffer with "q".
+
 ;; You are shown two tar files in tar-mode (see tar-mode for help).
 ;; In the case of old .deb format files, the control info is shown
 ;; but not the other files of control.tar, such as install scripts.
+;; Note that regular tar-mode commands e, f and RETURN show raw files
+;; without any special uncompressing or formatting.
 ;; Additional features that deb-view adds to tar-mode:
 ;; q - kill both view buffers (INFO and DATA) and return to the
 ;;     dired buffer if that's where you executed deb-mode.
@@ -106,13 +170,24 @@
 ;;      - fixed various spacing issues in doc strings.
 ;;      - disabled tar-mode keys that are not applicable to deb-view.
 
+;; 1.5  - added an auto-mode-alist and deb-view mode so that deb-view
+;;        is launched from any find-file command.
+;;      - added a deb-find command that takes a search string and creates
+;;        a buffer of matching deb files. ^C^C, RETURN or middle mouse button
+;;        runs deb-view on the selected deb file.
+;;      - added deb-view-help to "?" key in deb-view.
+
+;; 1.6  - improved doc strings for deb-find and deb-find-method.
+;;      - added (provide 'deb-view) and instructions for using
+;;        (require 'deb-view).
+;;      - reworked the documentation somewhat, but it's still too long.
+;;      - changed the copyright notice to refer to deb-view, not Emacs.
+
+;; 1.7  - make copy of compilation-minor-mode map rather than changing
+;;        it directly. It was breaking actual compilation buffer keymaps,
+;;        such as grep mode.
+
 
-;;; Code:
-
-(defvar deb-view-dired-view-return-buffer ""
-  "Return to this buffer after deb-view-dired-view-cleanup is called.")
-(make-variable-buffer-local 'deb-view-dired-view-return-buffer)
-
 ;; User variables:
 
 (defvar deb-view-tar-uncompress-program "gzip -cd"
@@ -123,7 +198,27 @@
 ;;  "*Non-nil means Tar mode should show the date/time of each subfile.
 ;; This information is useful, but it takes screen space away from file names.")
 
-;;; User functions:
+(defvar deb-find-method "find"
+  "Internal deb-find methods supported: locate or find.
+Any other entry is assumed to be an external command.
+See also the variable deb-find-directory.")
+
+(defvar deb-find-directory "/usr/local/src/debian"
+  "Directory to run find in when deb-find-method is \"find\".")
+
+;;; Code:
+
+(defvar deb-view-dired-view-return-buffer ""
+  "Return to this buffer after deb-view-dired-view-cleanup is called.")
+(make-variable-buffer-local 'deb-view-dired-view-return-buffer)
+
+;; You might not like the key bindings that I chose:
+(if (featurep 'dired)
+    (define-key dired-mode-map "\C-d" 'deb-view-dired-view)
+  (add-hook
+   'dired-load-hook
+   (function (lambda ()
+               (define-key dired-mode-map "\C-d" 'deb-view-dired-view)))))
 
 ;; You might not like the key bindings that I chose:
 (add-hook
@@ -140,6 +235,7 @@
              (define-key tar-mode-map "x"      'undefined)
              (define-key tar-mode-map ""     'undefined)
              
+             (define-key tar-mode-map "?"    'deb-view-help)
              (define-key tar-mode-map "q"    'deb-view-dired-view-cleanup)
              (define-key tar-mode-map "N"    'deb-view-tar-man)
              (define-key tar-mode-map "W"    'deb-view-tar-w3)
@@ -162,27 +258,9 @@ Press \"q\" in either window to kill both buffers and return to the dired buffer
 in either window to kill both buffers.
 
 In dired, press ^d on the dired line of the .deb file to view.
-Or, execute: ESC x deb-view RET, and enter the .deb file name
-at the prompt.
-
-You are shown two tar files in tar-mode (see tar-mode for help).
-In the case of old .deb format files, the control info is shown
-but not the other files of control.tar, such as install scripts.
-Additional features that deb-view adds to tar-mode:
-q - kill both view buffers (INFO and DATA) and return to the
-    dired buffer if that's where you executed deb-mode.
-v - executes deb-view-tar-view instead of tar-view, with the
-    additional smarts to uncompress .gz and .Z files for viewing.
-N - Like in dired, formats man pages for viewing, with the
-    additional smarts to uncompress .gz and .Z man files for viewing.
-W - use w3-mode to view an HTML file.
-These functions are also available in tar-mode on real tar files
-when deb-view is loaded.
-
-To view files not supported by deb-view, such as graphics, use the
-copy command in tar-mode (\"c\") to copy the file to a temp directory.
-You can then do what you want to the file. "
-  (interactive "fdeb file to view:")
+Or, execute: ESCAPE x deb-view RETURN, and enter the .deb file name
+at the prompt."
+  (interactive "fdeb file to view: ")
   (require 'view)
   (let* ((info-buffer-name (concat (file-name-nondirectory debfile) "-INFO"))
          (data-buffer-name (concat (file-name-nondirectory debfile) "-DATA"))
@@ -250,12 +328,65 @@ You can then do what you want to the file. "
     (set-buffer-modified-p nil)
     (setq buffer-read-only t)
     (switch-to-buffer-other-window data-buffer)
-    (if new-archive-format (other-window 1))))
+    (if new-archive-format (other-window 1))
+    (message "? for help. q to quit.")))
+
+(defun deb-view-mode ()
+  "View mode for Debian Archive Files."
+  (interactive)
+  (let ((deb-file buffer-file-name)
+        (return-buffer (nth 0 (buffer-list)))
+        (curbuf (current-buffer)))
+    (set-buffer return-buffer)
+    (kill-buffer curbuf)
+    ;; If you ever have a problem (with an emacs version newer than 19.34)
+    ;; with the "unread-command-events" method, use the deb-view call
+    ;; directly.
+    ;; (deb-view buffer)
+    (setq unread-command-events
+          (append "\exdeb-view" "\C-m" deb-file "\C-m"
+                  unread-command-events))))
+  
+(defun deb-find ()
+  "Search for deb files using the method specified by the variable
+deb-find-method, and collect output in a buffer.
+See also the variable deb-find-directory.
+
+This command uses a special history list, so you can
+easily repeat a deb-find command."
+  (interactive)
+  (require 'compile)
+  (let* ((deb-file-string (read-from-minibuffer "deb file to find: "
+                                         nil nil nil 'deb-find-history))
+         (output-buffer-name "*deb-find*")
+         (command (cond ((string-equal deb-find-method "locate")
+                         (concat "locate '" deb-file-string "' | egrep '\.deb$'"))
+                        ((string-equal deb-find-method "find")
+                         (concat "find " deb-find-directory " | egrep '"
+                                 deb-file-string "' | egrep '\.deb$'"))
+                        (t
+                         (concat deb-find-method " '" deb-file-string "'")))))
+    (compile-internal command "Not applicable in deb-find" "deb-find" nil nil
+                      (function (lambda (mode) output-buffer-name)))
+    (switch-to-buffer-other-window output-buffer-name)
+    (setq deb-view-find-minor-mode-map (copy-keymap compilation-minor-mode-map))
+    (use-local-map deb-view-find-minor-mode-map)
+    (define-key deb-view-find-minor-mode-map [mouse-2]  'deb-find-mouse-deb-view)
+    (define-key deb-view-find-minor-mode-map "\C-c\C-c" 'deb-find-deb-view)
+    (define-key deb-view-find-minor-mode-map "\C-m"     'deb-find-deb-view)
+    (define-key deb-view-find-minor-mode-map "?"        'deb-find-help)
+    (define-key deb-view-find-minor-mode-map "q"        'kill-this-buffer)
+    (define-key deb-view-find-minor-mode-map "\M-n" 'undefined)
+    (define-key deb-view-find-minor-mode-map "\M-p" 'undefined)
+    (define-key deb-view-find-minor-mode-map "\M-{" 'undefined)
+    (define-key deb-view-find-minor-mode-map "\M-}" 'undefined)
+    (beginning-of-buffer)
+    (message "? for help. q to quit.")))
 
 
 ;;; Internal functions:
 
-(defvar deb-view-version "1.4"
+(defvar deb-view-version "1.7"
   "The version of deb-view.")
 
 (defun deb-view-version ()
@@ -287,6 +418,58 @@ When called interactively, displays the version."
     (and (buffer-live-p ddir-buffer)
          (switch-to-buffer ddir-buffer))))
 
+(defun deb-find-help ()
+  "Show help information for deb-find."
+  (interactive)
+  (with-output-to-temp-buffer "*Help*"
+    (princ (format "deb-find mode:    version %s" (deb-view-version)))
+    (princ "\n
+RET     - view the deb file on this line with deb-view.
+C-c C-c - view the deb file on this line with deb-view.
+mouse-2 - view the deb file on this line with deb-view.
+? - show deb-find-help.
+q - quit deb-find.")
+    (save-excursion
+      (set-buffer standard-output)
+      (help-mode))
+    (print-help-return-message)))
+
+(defun deb-view-help ()
+  "Show help information for deb-view."
+  (interactive)
+  (with-output-to-temp-buffer "*Help*"
+    (princ (format "deb-view mode:    version %s" (deb-view-version)))
+    (princ "
+Based-on tar-mode, with additional features for viewing deb files.
+Execute \"^h m\" to see tar-mode bindings.
+
+You are shown two tar files in tar-mode (see tar-mode for help).
+In the case of old .deb format files, the control info is shown
+but not the other files of control.tar, such as install scripts.
+
+Note that regular tar-mode commands e, f and RETURN show raw files
+without any special uncompressing or formatting.
+
+Additional features that deb-view adds to tar-mode:
+? - show deb-view help.
+q - kill both view buffers (INFO and DATA) and return to the
+    dired buffer if that's where you executed deb-mode.
+v - executes deb-view-tar-view instead of tar-view, with the
+    additional smarts to uncompress .gz and .Z files for viewing.
+N - Like in dired, formats man pages for viewing, with the
+    additional smarts to uncompress .gz and .Z man files for viewing.
+W - use w3-mode to view an HTML file.
+These functions are also available in tar-mode on normal tar files
+when deb-view is loaded.
+
+To view files not supported by deb-view, such as graphics, use the
+copy command in tar-mode (\"c\") to copy the file to a temp directory.
+You can then do what you want to the file.")
+    (save-excursion
+      (set-buffer standard-output)
+      (help-mode))
+    (print-help-return-message)))
+
 (defun deb-view-tar-man ()
   "*In Tar mode, view the tar file entry on this line as a man page."
   (interactive)
@@ -349,4 +532,30 @@ It then selects a major mode from the uncompressed file name and contents.
 	(if w3-mutable-windows (pop-to-buffer x) (switch-to-buffer x))))
   (view-exit))
 
+(defvar deb-find-history nil
+  "History list for deb-find commands.")
+
+(defvar deb-find-regexp "^/.*\.deb$"
+  "Regexp for deb file names in the deb-find buffer.")
+
+(defun deb-find-deb-view ()
+  (interactive)
+  (let ((deb-file (thing-at-point 'filename)))
+    (if (and deb-file
+             (string-match deb-find-regexp deb-file))
+        (deb-view (thing-at-point 'filename))
+      (error "No deb file on this line."))))
+
+(defun deb-find-mouse-deb-view (event)
+  (interactive "e")
+  (pop-to-buffer (window-buffer (posn-window (event-end event))))
+  (goto-char (posn-point (event-end event)))
+  (let ((deb-file (thing-at-point 'filename)))
+    (if (and deb-file
+             (string-match deb-find-regexp deb-file))
+        (deb-view (thing-at-point 'filename))
+      (error "No deb file on this line."))))
+
+(provide 'deb-view)
+
 ;;; deb-view.el ends here
